/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataTypes;

public class Feature implements Comparable{

	private String type;
	private Location location = null;
	private String name = "None";
	private String source = "none";
	private StringBuffer annotation = new StringBuffer();
	private Chromosome chromosome;
	private String description = "";
	
	public Feature (String type, Chromosome chromosome) {
		this.type = new String(type);
		this.chromosome = chromosome;
	}
	
	public void setLocation (Location location) {
		this.location = location;
		name = location.start()+".."+location.end();
		source = "location";

	}
	
	public void setName (String name) {
		this.name = new String(name);
		source = "manual";
	}
	
	public Chromosome chromosome () {
		return chromosome;
	}
	
	public String getAllAnnotation (){
		return annotation.toString();
	}
		
	public void addAttribute (String key, String value) {
		
		// We tried storing attributes in a more structured way, but
		// the memory overhead was too big.  We therefore just put them
		// all into one big string buffer.  Before we do that we check
		// the new data to see if it provides a better name for the 
		// feature than the one we already have.
		
		if (key.startsWith("/")) {
			key = key.substring(1);
		}
		
		findBestName(key,value);

		annotation.append(key);
		for (int i=0;i<15-key.length();i++) 
			annotation.append(" ");
		
		annotation.append(value);
		annotation.append("\n");
		
		if (key.equals("description")) {
			description = new String(value);
		}
	}
	
	public String type () {
		return type;
	}
	
	public Location location () {
		return location;
	}
	
	public String name () {
		name =name.replaceAll("\"","");
		return name;
	}
	
	public String description () {
		return description;
	}
	
	public String toString() {
		return name();
	}
	
	private void findBestName (String key, String value) {
		
		// We need to see if a better name than the one we've currently
		// got has been found already.

		// Manual naming always trumps everything, so use that if it's there
		if (source.equals("manual")) return;
		
		// A proper name attribute is the next best option.
		if (source.equals("name")) return;
		if (key.equals("name")) {
			name = new String(value);
			source="name";
			return;
		}
		
		if (source.equals("db_xref")) return;
		
		if (key.equals("db_xref")) {
			if (value.indexOf("MarkerSymbol")>=0) {
				name = new String((value.split(":"))[1]);
				source = "db_xref";
				return;
			}
		}
		
		if (source.equals("gene")) return;
		
		// Most things have a gene attribute
		if (key.equals("gene")){
			name = new String(value);
			source = "gene";
			return;
		}
		
		if (source.equals("exon_id")) return;
		// Exons have an exon id
		if (key.equals("note")) {
			if (value.indexOf("exon_id")>=0) {
				name = new String((value.split("="))[1]);
				source="exon_id";
				return;
			}
		}
		
		if (source.equals("standard_name")) return;
		
		// STSs have a "standard name"
		if (key.equals("standard_name")){
			name = new String(value);
			source = "standard_name";
			return;
		}
		
		if (source.equals("note")) return;
		
		// We can use the first note for misc_features
		if (key.equals("note")){
			name = new String(value);
			source = "note";
			return;
		}
				
	}

	public int compareTo(Object o) {
		return name.compareTo(o.toString());
	}
	
}
