/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataTypes;

import java.util.Arrays;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;

public class Location {

	private int start=0;
	private int end=0;
	private int strand;
	private SubLocation [] subLocations;
	
	public static final int FORWARD = 1;
	public static final int REVERSE = 2;
	public static final int NONE = 3;
	
	public Location (String EMBLString) throws ChipMonkException {
		this(EMBLString,0);
	}
	
	public Location (String EMBLString, int offset) throws ChipMonkException {
		if (EMBLString.indexOf("complement")>=0) {
			strand = REVERSE;
		}
		else {
			strand = FORWARD;
		}
		
		EMBLString = EMBLString.replaceAll("join\\(","");
		EMBLString = EMBLString.replaceAll("complement\\(","");
		EMBLString = EMBLString.replaceAll("\\)","");
		
		// We need to check if any of the positions contain > or < symbols
		// (used for undefined positions).  If so then we remove these.
		EMBLString = EMBLString.replaceAll(">", "");
		EMBLString = EMBLString.replaceAll("<", "");
		
		String [] subLocationStrings = EMBLString.split(",");
		subLocations = new SubLocation[subLocationStrings.length];
		
		for (int i=0;i<subLocationStrings.length;i++) {
			String [] positions = subLocationStrings[i].split("\\.\\.");
						
			// Some EMBL files specify a single base position.  In these
			// cases we just put the location as that base to that base +1
			if (positions.length == 2) {
				subLocations[i] = new SubLocation(Integer.parseInt(positions[0])+offset,Integer.parseInt(positions[1])+offset);
			}
			else if (positions.length == 1) {
				subLocations[i] = new SubLocation(Integer.parseInt(positions[0])+offset,Integer.parseInt(positions[0])+offset+1);				
			}
			if (start == 0) {
				start = subLocations[i].start();
				end = subLocations[i].end();
			}
			else {
				if (subLocations[i].start()< start) {
					start = subLocations[i].start();					
				}
				if (subLocations[i].end() > end) {
					end = subLocations[i].end();
				}
			}
		}
		Arrays.sort(subLocations);
	}
	
	public SubLocation [] subLocations () {
		return subLocations;
	}
	
	public int start () {
		return start;
	}
	
	public int end () {
		return end;
	}
	
	public int strand () {
		return strand;
	}
	
}
