package uk.ac.bbsrc.babraham.ChipMonk.Displays.ConditionTree;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class ConditionTree extends JDialog  implements MatrixProgressListener, ActionListener {
	
	private ConditionTreeNode outgroup = null;
	private float maxDistance = 0;
	private int leafCount = 0;
	private Vector leafNodes = new Vector();
	private ChipMonkApplication application;
	private ConditionTreePanel conditionTreePanel;
	
	
	public ConditionTree (ChipMonkApplication application) {
		super(application);
		this.application = application;
		
		// Do some sanity checks
		if (application.dataCollection().getAllDataSets().length < 3) {
			JOptionPane.showMessageDialog(application,"At least three data sets are needed to construct a tree","Error",JOptionPane.ERROR_MESSAGE);
			this.dispose();
			return;
		}
		
		DistanceMatrix matrix = new DistanceMatrix();
		matrix.addListener(this);
		matrix.calculateTree(application.dataCollection().getAllDataSets(),application.dataCollection().probeSet().getActiveProbes());
			
		setTitle("Condition Tree...");
		getContentPane().setLayout(new BorderLayout());
		conditionTreePanel = new ConditionTreePanel();
		getContentPane().add(conditionTreePanel,BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("close");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		JButton saveButton = new JButton("Save");
		saveButton.setActionCommand("save");
		saveButton.addActionListener(this);
		buttonPanel.add(saveButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setSize(500,500);
		setLocationRelativeTo(application);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setVisible(true);

	}

	private void getLeafCount () {
		ConditionTreeNode rootNode = (ConditionTreeNode)outgroup.getParent();
		getLeafCount(rootNode);
		maxDistance = findMaxDistance();
	}
	
	private float findMaxDistance () {
		
		float max = 0;
		Enumeration e = leafNodes.elements();
		while (e.hasMoreElements()) {
			float f = ((ConditionTreeNode)e.nextElement()).getDistanceFromRoot();
			if (f>max) {
				max = f;
			}
		}
		return max;
	}

	private void getLeafCount (ConditionTreeNode n) {
		if (n.isLeaf()) {
			leafNodes.add(n);
			++leafCount;
		}
		else {
			Enumeration e = n.children();
			while (e.hasMoreElements()) {
				ConditionTreeNode n2 = (ConditionTreeNode)e.nextElement();
				getLeafCount(n2);
			}
		}
	}
	
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("close")){
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("save")){
			JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getSaveLocation());
			chooser.setMultiSelectionEnabled(false);
			chooser.setFileFilter(new FileFilter() {
				
				public String getDescription() {
					return "PNG Files";
				}
			
				public boolean accept(File f) {
					if (f.isDirectory() || f.getName().toLowerCase().endsWith(".png")) {
						return true;
					}
					else {
						return false;
					}
				}
			
			});
			
			int result = chooser.showSaveDialog(this);
			if (result == JFileChooser.CANCEL_OPTION) return;
	
			File file = chooser.getSelectedFile();
			if (! file.getPath().toLowerCase().endsWith(".png")) {
				file = new File(file.getPath()+".png");
			}
	
			// Check if we're stepping on anyone's toes...
			if (file.exists()) {
				int answer = JOptionPane.showOptionDialog(this,file.getName()+" exists.  Do you want to overwrite the existing file?","Overwrite file?",0,JOptionPane.QUESTION_MESSAGE,null,new String [] {"Overwrite and Save","Cancel"},"Overwrite and Save");

				if (answer > 0) {
					return;
				}
			}

			
			try {					
				ImageIO.write((BufferedImage)(conditionTreePanel.getCurrentView()),"PNG",file);
			}
	
			catch (IOException e) {
				JOptionPane.showMessageDialog(this,"Error saving image: "+e.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	private class ConditionTreePanel extends JPanel {
		
		private final int xGap=5;
		private final int yGap=5;
		private final float treeProportion = 0.75f; // The proportion of the visible area used to draw the tree - the remainder is for labels.
		private int width;
		private int height;
		private int leafHeight;
		
		public ConditionTreePanel () {
//			Enumeration e = leafNodes.elements();
//			while (e.hasMoreElements()) {
//				System.out.println("Node "+e.nextElement());
//			}
		}
		
		public void paintComponent (Graphics g) {
			super.paintComponent(g);
			width = getWidth();
			height = getHeight();
			leafHeight = (height - (2*yGap))/(leafNodes.size()+1);
			g.setColor(Color.WHITE);
			g.fillRect(0,0,width,height);
			g.setColor(Color.BLACK);
			
			String probeSetName = application.dataCollection().probeSet().getActiveProbeListName();
			if (probeSetName==null) {
				probeSetName="All Probes";
			}
			
			g.drawString("Condition Tree using "+probeSetName,xGap,15);
			
			if (outgroup == null) {
				g.drawString("Calculating Tree...",xGap,height/2);
			}
			else {
				drawNode((ConditionTreeNode)(outgroup.getParent()),yGap,height-yGap,g);
			}
		}
		
		public Image getCurrentView() {
			BufferedImage b = new BufferedImage(getWidth(),getHeight(),BufferedImage.TYPE_INT_RGB);
			Graphics g = b.getGraphics();			
			this.paint(g);
			return b;
		}
		
		private void drawNode (ConditionTreeNode n, int yTop, int yBottom, Graphics g) {
			
//			System.out.println("Drawing root node "+n+" between "+yBottom+" and "+yTop);
			
			// This is an interative method.  We draw the connecting 
			// lines for this node.  We also draw any leaf node
			// children it has.  If we have other branch node children
			// then we call ourselves again and start the dance all over.
			
			ConditionTreeNode [] children = new ConditionTreeNode [n.getChildCount()];
			Enumeration c = n.children();
			int cc=0;
			while (c.hasMoreElements()) {
				children[cc] = (ConditionTreeNode)c.nextElement();
				cc++;
			}
			
			// We sort the children so we're always drawing from the smallest 
			// to the largest
			Arrays.sort(children);
			int bottomLineHeight = yBottom;
			int topLineHeight = yTop;
			
			int childrenSeenSoFar = 0;
			for (int i=0;i<children.length;i++) {
				
				int midLine;				
				
				
				int totalOffspring;
				if (children[i].isLeaf()) {
					totalOffspring = 1;
				}
				else {
					totalOffspring = children[i].getTotalChildCount();
				}
				
				int thisBlockStart = yBottom-(leafHeight*childrenSeenSoFar);
				int thisBlockEnd = thisBlockStart-(leafHeight*totalOffspring);
				
//				System.out.println("Current block goes between "+thisBlockStart+" and "+thisBlockEnd);
				
				
				// We need to know how far up the block we need to start
				// and end the vertical line.  This is a function of the
				// number of offspring of our first and last children
				// and the arrangement of their kids. Aaaarrrgh!
				
				// The easy option is that this block is a leaf
				if (children[i].isLeaf()) {
					midLine = thisBlockStart-(leafHeight/2);
				}
				
				// If we have grandkids then things are a bit more complicated
				else {
				
					ConditionTreeNode [] grandkids = new ConditionTreeNode[children[i].getChildCount()];
					Enumeration o = children[i].children();
					int oo=0;
					while (o.hasMoreElements()) {
						grandkids[oo] = (ConditionTreeNode)o.nextElement();
						oo++;
					}
					Arrays.sort(grandkids);
					
					// We now need to figure out where the midline for the group is
					// to go.  This is a function of the total number of offspring for
					// this node, and the number of total offspring for the first and
					// last grandkids.  This makes my head hurt now so God help anyone
					// who has to come back later and understand this.  I'm really sorry.
					int bottomOfMid = thisBlockStart-(int)((grandkids[0].getTotalChildCount()/2f)*leafHeight);
					int topOfMid = thisBlockEnd+(int)((grandkids[grandkids.length-1].getTotalChildCount()/2f)*leafHeight);
					midLine = bottomOfMid-((bottomOfMid-topOfMid)/2);
					
				}
				
				// Now we can draw the midline:
				g.drawLine(getXForDistance(n.getDistanceFromRoot()),midLine,getXForDistance(children[i].getDistanceFromRoot()),midLine);
				
				// If we're a leaf draw a label
				if (children[i].isLeaf()) {
					g.drawString(children[i].toString(),getXForDistance(children[i].getDistanceFromRoot())+3,midLine+5);
				}
				// If we're another node then go draw us
				else {
//					System.out.println("Setting off new node.  Our distance to root is "+n.getDistanceFromRoot()+" new nodes distance is "+children[i].getDistanceFromRoot());
					drawNode(children[i],thisBlockStart-(totalOffspring*leafHeight),thisBlockStart,g);
				}
				childrenSeenSoFar += totalOffspring;
				if (i==0) {
					bottomLineHeight = midLine;
				}
				if (i==(children.length-1)) {
					topLineHeight = midLine;
				}
			}	
			g.drawLine(getXForDistance(n.getDistanceFromRoot()),bottomLineHeight,getXForDistance(n.getDistanceFromRoot()),topLineHeight);
		}
			
		
		private int getXForDistance (float d) {
//			System.out.println("Asked for distance for "+d);
			int x = xGap + (int)(((width-(2*xGap))*treeProportion)*(d/maxDistance));
//			System.out.println ("Answer was "+x);
			return x;
		}
		
	}


	public void matrixUpdateProgress(String message, int percentComlete) {
	}

	public void matrixExceptionReceived(Exception e) {
		//TODO: Put up an error here.
	}

	public void matrixProcessingFinished(ConditionTreeNode n) {
		outgroup = n;
		getLeafCount();
		findMaxDistance();

		this.repaint();
	}
}
