/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Normalisation;

import java.util.Enumeration;
import java.util.Vector;

import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataCollection;

// This class is a convenience module to help the running
// of multiple normalisation modules (each of which runs
// in a separate thread.  It is not a normal normalisation
// module and shouldn't be offered in the list of choices
// for normalisation.

public class NormalisationRunner implements Runnable, Normalisation, NormalisationProgressListener {

	private Normalisation [] modules;
	private DataCollection data;
	private Vector listeners = new Vector();
	private int lastLaunched = 0;
	
	public NormalisationRunner (Normalisation [] modules) {
		this.modules = modules;
		for (int i=0;i<modules.length;i++) {
			modules[i].addNormalisationProgressListener(this);
		}
	}
	
	public void normalise(DataCollection d) {
		if (modules == null || modules.length == 0) {
			normalisationComplete();
			return;
		}
		data = d;
		Thread t = new Thread(this);
		t.start();
	}

	public void addNormalisationProgressListener(NormalisationProgressListener l) {
		listeners.add(l);
	}

	public boolean isSuitable(DataCollection d) {
		return false;
	}

	public void run() {
		lastLaunched = 0;
		modules[0].normalise(data);
	
	}

	public void updateNormalisationProgress(String message) {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((NormalisationProgressListener)e.nextElement()).updateNormalisationProgress(message);
		}
	}

	public void normalisationComplete() {
		if (lastLaunched == (modules.length-1)) {
			Enumeration e = listeners.elements();
			while (e.hasMoreElements()) {
				((NormalisationProgressListener)e.nextElement()).normalisationComplete();
			}
		}
		else {
			lastLaunched++;
			modules[lastLaunched].normalise(data);
		}
	}

	public void normalisationExceptionReceived(Exception ex) {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((NormalisationProgressListener)e.nextElement()).normalisationExceptionReceived(ex);
		}
	}

	public String shortDescription() {
		return null;
	}

	public String longDescription() {
		return null;
	}
}
